@@ -828,7 +828,6 @@ func TestMounterSetUpWithFSGroup(t *testing.T) {
 	for i, tc := range testCases {
 		t.Logf("Running test %s", tc.name)
 
-		defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIVolumeFSGroupPolicy, tc.driverFSGroupPolicy)()
 		defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.DelegateFSGroupToCSIDriver, tc.delegateFSGroupFeatureGate)()
 
 		volName := fmt.Sprintf("test-vol-%d", i)
@@ -1202,6 +1201,24 @@ func Test_csiMountMgr_supportsFSGroup(t *testing.T) {
 			},
 			want: true,
 		},
+		{
+			name: "driverPolicy is ReadWriteOnceWithFSTypeFSGroupPolicy with CSI inline volume",
+			args: args{
+				fsGroup:      new(int64),
+				fsType:       "ext4",
+				driverPolicy: storage.ReadWriteOnceWithFSTypeFSGroupPolicy,
+			},
+			fields: fields{
+				spec: volume.NewSpecFromVolume(&api.Volume{
+					VolumeSource: api.VolumeSource{
+						CSI: &api.CSIVolumeSource{
+							Driver: testDriver,
+						},
+					},
+				}),
+			},
+			want: true,
+		},
 	}
 
 	for _, tt := range tests {
