@@ -43,6 +43,7 @@ import (
 	"k8s.io/kubernetes/test/e2e/storage/utils"
 	storageutils "k8s.io/kubernetes/test/e2e/storage/utils"
 	imageutils "k8s.io/kubernetes/test/utils/image"
+	admissionapi "k8s.io/pod-security-admission/api"
 )
 
 var (
@@ -116,6 +117,7 @@ func (s *subPathTestSuite) DefineTests(driver storageframework.TestDriver, patte
 	// Beware that it also registers an AfterEach which renders f unusable. Any code using
 	// f must run inside an It or Context callback.
 	f := framework.NewFrameworkWithCustomTimeouts("provisioning", storageframework.GetDriverTimeouts(driver))
+	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
 
 	init := func() {
 		l = local{}
@@ -882,7 +884,7 @@ func testPodContainerRestart(f *framework.Framework, pod *v1.Pod) {
 	testPodContainerRestartWithHooks(f, pod, &podContainerRestartHooks{
 		AddLivenessProbeFunc: func(p *v1.Pod, probeFilePath string) {
 			p.Spec.Containers[0].LivenessProbe = &v1.Probe{
-				Handler: v1.Handler{
+				ProbeHandler: v1.ProbeHandler{
 					Exec: &v1.ExecAction{
 						Command: []string{"cat", probeFilePath},
 					},
@@ -936,7 +938,7 @@ func TestPodContainerRestartWithConfigmapModified(f *framework.Framework, origin
 	testPodContainerRestartWithHooks(f, pod, &podContainerRestartHooks{
 		AddLivenessProbeFunc: func(p *v1.Pod, probeFilePath string) {
 			p.Spec.Containers[0].LivenessProbe = &v1.Probe{
-				Handler: v1.Handler{
+				ProbeHandler: v1.ProbeHandler{
 					Exec: &v1.ExecAction{
 						// Expect probe file exist or configmap mounted file has been modified.
 						Command: []string{"sh", "-c", fmt.Sprintf("cat %s || test `cat %s` = '%s'", probeFilePath, volumePath, modifiedValue)},
