@@ -14,44 +14,48 @@ See the License for the specific language governing permissions and
 limitations under the License.
 */
 
-package e2e_node
+package e2enode
 
 import (
-	"k8s.io/kubernetes/pkg/api"
-	"k8s.io/kubernetes/pkg/kubelet/dockertools"
+	"context"
+	"k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/kubernetes/test/e2e/framework"
+	e2epod "k8s.io/kubernetes/test/e2e/framework/pod"
+	admissionapi "k8s.io/pod-security-admission/api"
 
 	"github.com/davecgh/go-spew/spew"
-	. "github.com/onsi/ginkgo"
-	. "github.com/onsi/gomega"
+	"github.com/onsi/ginkgo"
+	"github.com/onsi/gomega"
 )
 
-var _ = framework.KubeDescribe("ImageID", func() {
+var _ = SIGDescribe("ImageID [NodeFeature: ImageID]", func() {
 
-	busyBoxImage := "gcr.io/google_containers/busybox@sha256:4bdd623e848417d96127e16037743f0cd8b528c026e9175e22a84f639eca58ff"
+	busyBoxImage := "k8s.gcr.io/busybox@sha256:4bdd623e848417d96127e16037743f0cd8b528c026e9175e22a84f639eca58ff"
 
 	f := framework.NewDefaultFramework("image-id-test")
+	f.NamespacePodSecurityEnforceLevel = admissionapi.LevelPrivileged
 
-	It("should be set to the manifest digest (from RepoDigests) when available", func() {
-		podDesc := &api.Pod{
-			ObjectMeta: api.ObjectMeta{
+	ginkgo.It("should be set to the manifest digest (from RepoDigests) when available", func() {
+		podDesc := &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
 				Name: "pod-with-repodigest",
 			},
-			Spec: api.PodSpec{
-				Containers: []api.Container{{
+			Spec: v1.PodSpec{
+				Containers: []v1.Container{{
 					Name:    "test",
 					Image:   busyBoxImage,
 					Command: []string{"sh"},
 				}},
-				RestartPolicy: api.RestartPolicyNever,
+				RestartPolicy: v1.RestartPolicyNever,
 			},
 		}
 
 		pod := f.PodClient().Create(podDesc)
 
-		framework.ExpectNoError(framework.WaitTimeoutForPodNoLongerRunningInNamespace(
-			f.ClientSet, pod.Name, f.Namespace.Name, "", framework.PodStartTimeout))
-		runningPod, err := f.PodClient().Get(pod.Name)
+		framework.ExpectNoError(e2epod.WaitTimeoutForPodNoLongerRunningInNamespace(
+			f.ClientSet, pod.Name, f.Namespace.Name, framework.PodStartTimeout))
+		runningPod, err := f.PodClient().Get(context.TODO(), pod.Name, metav1.GetOptions{})
 		framework.ExpectNoError(err)
 
 		status := runningPod.Status
@@ -61,6 +65,6 @@ var _ = framework.KubeDescribe("ImageID", func() {
 			return
 		}
 
-		Expect(status.ContainerStatuses[0].ImageID).To(Equal(dockertools.DockerPullablePrefix + busyBoxImage))
+		gomega.Expect(status.ContainerStatuses[0].ImageID).To(gomega.ContainSubstring(busyBoxImage))
 	})
 })
